# Copyright (c) Semi-ATE
# Distributed under the terms of the MIT License
import os

# Constants
HERE = os.path.abspath(os.path.dirname(__file__))
CONFIG_TEMPLATE = "config.jinja2"
CONFIG_TEMPLATE_PATH = os.path.join(HERE, "config.jinja2")
